@@ -113,9 +113,8 @@ mod filters {
             FFIType::Int64 | FFIType::UInt64 => "Long".to_string(),
             FFIType::Float32 => "Float".to_string(),
             FFIType::Float64 => "Double".to_string(),
-            FFIType::RustCString => "Pointer".to_string(),
+            FFIType::RustArcPtr => "Pointer".to_string(),
             FFIType::RustBuffer => "RustBuffer.ByValue".to_string(),
-            FFIType::RustError => "RustError".to_string(),
             FFIType::ForeignBytes => "ForeignBytes.ByValue".to_string(),
             FFIType::ForeignCallback => "ForeignCallback".to_string(),
         })
@@ -184,6 +183,23 @@ mod filters {
         Ok(nm.to_string().to_shouty_snake_case())
     }
 
+    /// Get the idiomatic Kotlin rendering of an exception name
+    ///
+    /// This replaces "Error" at the end of the name with "Exception".  Rust code typically uses
+    /// "Error" for any type of error but in the Java world, "Error" means a non-recoverable error
+    /// and is distinguished from an "Exception".
+    pub fn exception_name_kt(nm: &dyn fmt::Display) -> Result<String, askama::Error> {
+        let name = nm.to_string();
+        match name.strip_suffix("Error") {
+            None => Ok(name),
+            Some(stripped) => {
+                let mut kt_exc_name = stripped.to_owned();
+                kt_exc_name.push_str("Exception");
+                Ok(kt_exc_name)
+            }
+        }
+    }
+
     /// Get a Kotlin expression for lowering a value into something we can pass over the FFI.
     ///
     /// Where possible, this delegates to a `lower()` method on the type itself, but special
