@@ -35,7 +35,8 @@ enum class {{ e.name()|class_name_kt }} {
 
 {% else %}
 
-sealed class {{ e.name()|class_name_kt }} {
+{% call kt::unsigned_types_annotation(e) %}
+sealed class {{ e.name()|class_name_kt }}{% if ci.item_contains_object_references(e) %}: Disposable {% endif %} {
     {% for variant in e.variants() -%}
     {% if !variant.has_fields() -%}
     object {{ variant.name()|class_name_kt }} : {{ e.name()|class_name_kt }}()
@@ -78,11 +79,28 @@ sealed class {{ e.name()|class_name_kt }} {
                 buf.putInt({{ loop.index }})
                 {% for field in variant.fields() -%}
                 {{ "(this.{})"|format(field.name())|write_kt("buf", field.type_()) }}
-                {% endfor -%}
+                {% endfor %}
             }
             {%- endfor %}
         }.let { /* this makes the `when` an expression, which ensures it is exhaustive */ }
     }
+
+    {% if ci.item_contains_object_references(e) %}
+    @Suppress("UNNECESSARY_SAFE_CALL") // codegen is much simpler if we unconditionally emit safe calls here
+    override fun destroy() {
+        when(this) {
+            {%- for variant in e.variants() %}
+            is {{ e.name()|class_name_kt }}.{{ variant.name()|class_name_kt }} -> {
+                {% for field in variant.fields() -%}
+                    {%- if ci.item_contains_object_references(field) -%}
+                    this.{{ field.name() }}?.destroy()
+                    {% endif -%}
+                {%- endfor %}
+            }
+            {%- endfor %}
+        }.let { /* this makes the `when` an expression, which ensures it is exhaustive */ }
+    }
+    {% endif %}
 }
 
 {% endif %}
