@@ -1,3 +1,4 @@
+{% call kt::unsigned_types_annotation(obj) %}
 public interface {{ obj.name()|class_name_kt }}Interface {
     {% for meth in obj.methods() -%}
     fun {{ meth.name()|fn_name_kt }}({% call kt::arg_list_decl(meth) %})
@@ -8,9 +9,10 @@ public interface {{ obj.name()|class_name_kt }}Interface {
     {% endfor %}
 }
 
+{% call kt::unsigned_types_annotation(obj) %}
 class {{ obj.name()|class_name_kt }}(
-    handle: Long
-) : FFIObject(handle), {{ obj.name()|class_name_kt }}Interface {
+    pointer: Pointer
+) : FFIObject(pointer), {{ obj.name()|class_name_kt }}Interface {
 
     {%- match obj.primary_constructor() %}
     {%- when Some with (cons) %}
@@ -22,23 +24,31 @@ class {{ obj.name()|class_name_kt }}(
     /**
      * Disconnect the object from the underlying Rust object.
      * 
-     * It can be called more than once, but once called, interacting with the object 
+     * It can be called more than once, but once called, interacting with the object
      * causes an `IllegalStateException`.
      * 
      * Clients **must** call this method once done with the object, or cause a memory leak.
      */
-    override protected fun freeHandle() {
-        rustCall(InternalError.ByReference()) { err ->
-            _UniFFILib.INSTANCE.{{ obj.ffi_object_free().name() }}(this.handle, err)
+    override protected fun freeRustArcPtr() {
+        rustCall() { status ->
+            _UniFFILib.INSTANCE.{{ obj.ffi_object_free().name() }}(this.pointer, status)
         }
     }
 
+    internal fun lower(): Pointer = callWithPointer { it }
+
+    internal fun write(buf: RustBufferBuilder) {
+        // The Rust code always expects pointers written as 8 bytes,
+        // and will fail to compile if they don't fit.
+        buf.putLong(Pointer.nativeValue(this.lower()))
+    }
+
     {% for meth in obj.methods() -%}
     {%- match meth.return_type() -%}
 
     {%- when Some with (return_type) -%}
     override fun {{ meth.name()|fn_name_kt }}({% call kt::arg_list_protocol(meth) %}): {{ return_type|type_kt }} =
-        callWithHandle {
+        callWithPointer {
             {%- call kt::to_ffi_call_with_prefix("it", meth) %}
         }.let {
             {{ "it"|lift_kt(return_type) }}
@@ -46,18 +56,26 @@ class {{ obj.name()|class_name_kt }}(
 
     {%- when None -%}
     override fun {{ meth.name()|fn_name_kt }}({% call kt::arg_list_protocol(meth) %}) =
-        callWithHandle {
+        callWithPointer {
             {%- call kt::to_ffi_call_with_prefix("it", meth) %}
         }
     {% endmatch %}
     {% endfor %}
 
-    {% if obj.constructors().len() > 1 -%}
     companion object {
+        internal fun lift(ptr: Pointer): {{ obj.name()|class_name_kt }} {
+            return {{ obj.name()|class_name_kt }}(ptr)
+        }
+
+        internal fun read(buf: ByteBuffer): {{ obj.name()|class_name_kt }} {
+            // The Rust code always writes pointers as 8 bytes, and will
+            // fail to compile if they don't fit.
+            return {{ obj.name()|class_name_kt }}.lift(Pointer(buf.getLong()))
+        }
+
         {% for cons in obj.alternate_constructors() -%}
         fun {{ cons.name()|fn_name_kt }}({% call kt::arg_list_decl(cons) %}): {{ obj.name()|class_name_kt }} =
             {{ obj.name()|class_name_kt }}({% call kt::to_ffi_call(cons) %})
         {% endfor %}
     }
-    {%- endif %}
 }
