@@ -1,3 +1,4 @@
+{% call kt::unsigned_types_annotation(rec) %}
 data class {{ rec.name()|class_name_kt }} (
     {%- for field in rec.fields() %}
     var {{ field.name()|var_name_kt }}: {{ field.type_()|type_kt -}}
@@ -7,9 +8,8 @@ data class {{ rec.name()|class_name_kt }} (
     {%- endmatch -%}
     {% if !loop.last %}, {% endif %}
     {%- endfor %}
-) {
+) {% if ci.item_contains_object_references(rec) %}: Disposable {% endif %}{
     companion object {
-        // XXX TODO: put this in a superclass maybe?
         internal fun lift(rbuf: RustBuffer.ByValue): {{ rec.name()|class_name_kt }} {
             return liftFromRustBuffer(rbuf) { buf -> {{ rec.name()|class_name_kt }}.read(buf) }
         }
@@ -30,6 +30,17 @@ data class {{ rec.name()|class_name_kt }} (
     internal fun write(buf: RustBufferBuilder) {
         {%- for field in rec.fields() %}
             {{ "(this.{})"|format(field.name())|write_kt("buf", field.type_()) }}
+        {% endfor %}
+    }
+
+    {% if ci.item_contains_object_references(rec) %}
+    @Suppress("UNNECESSARY_SAFE_CALL") // codegen is much simpler if we unconditionally emit safe calls here
+    override fun destroy() {
+        {% for field in rec.fields() %}
+            {%- if ci.item_contains_object_references(field) -%}
+            this.{{ field.name() }}?.destroy()
+            {% endif -%}
         {%- endfor %}
     }
-}
\ No newline at end of file
+    {% endif %}
+}
