@@ -62,7 +62,7 @@ class RustBufferBuilder(object):
     {% when Type::UInt16 -%}
 
     def writeU16(self, v):
-        self._pack_into(1, ">H", v)
+        self._pack_into(2, ">H", v)
 
     {% when Type::Int32 -%}
 
@@ -109,12 +109,12 @@ class RustBufferBuilder(object):
     {% when Type::Timestamp -%}
 
     def write{{ canonical_type_name }}(self, v):
-        if v >= datetime.datetime.fromtimestamp(0):
+        if v >= datetime.datetime.fromtimestamp(0, datetime.timezone.utc):
             sign = 1
-            delta = v - datetime.datetime.fromtimestamp(0)
+            delta = v - datetime.datetime.fromtimestamp(0, datetime.timezone.utc)
         else:
             sign = -1
-            delta = datetime.datetime.fromtimestamp(0) - v
+            delta = datetime.datetime.fromtimestamp(0, datetime.timezone.utc) - v
 
         seconds = delta.seconds + delta.days * 24 * 3600
         nanoseconds = delta.microseconds * 1000
@@ -133,24 +133,14 @@ class RustBufferBuilder(object):
 
     {% when Type::Object with (object_name) -%}
     # The Object type {{ object_name }}.
-    # Objects cannot currently be serialized, but we can produce a helpful error.
+    # We write the pointer value directly - what could possibly go wrong?
 
-    def write{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.write() not implemented yet for {{ canonical_type_name }}")
-
-    {% when Type::CallbackInterface with (object_name) -%}
-    # The Callback Interface type {{ object_name }}.
-    # Objects cannot currently be serialized, but we can produce a helpful error.
-
-    def write{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.write() not implemented yet for {{ canonical_type_name }}")
-
-    {% when Type::Error with (error_name) -%}
-    # The Error type {{ error_name }}.
-    # Errors cannot currently be serialized, but we can produce a helpful error.
-
-    def write{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.write() not implemented yet for {{ canonical_type_name }}")
+    def write{{ canonical_type_name }}(self, v):
+        if not isinstance(v, {{ object_name|class_name_py }}):
+            raise TypeError("Expected {{ object_name|class_name_py }} instance, {} found".format(v.__class__.__name__))
+        # The Rust code always expects pointers written as 8 bytes,
+        # and will fail to compile if they don't fit in that size.
+        self.writeU64(v._pointer)
 
     {% when Type::Enum with (enum_name) -%}
     {%- let e = ci.get_enum_definition(enum_name).unwrap() -%}
@@ -205,5 +195,11 @@ class RustBufferBuilder(object):
             self.writeString(k)
             self.write{{ inner_type.canonical_name()|class_name_py }}(v)
 
+    {%- else -%}
+    # This type cannot currently be serialized, but we can produce a helpful error.
+
+    def write{{ canonical_type_name }}(self, value):
+        raise InternalError("RustBufferStream.write() not implemented yet for {{ canonical_type_name }}")
+
     {%- endmatch -%}
-    {%- endfor %}
\ No newline at end of file
+    {%- endfor %}
