@@ -51,7 +51,7 @@ class RustBufferStream(object):
     {% when Type::UInt16 -%}
 
     def readU16(self):
-        return self._unpack_from(1, ">H")
+        return self._unpack_from(2, ">H")
 
     {% when Type::Int32 -%}
 
@@ -113,10 +113,13 @@ class RustBufferStream(object):
     def read{{ canonical_type_name }}(self):
         seconds = self._unpack_from(8, ">q")
         microseconds = self._unpack_from(4, ">I") / 1000
+        # Use fromtimestamp(0) then add the seconds using a timedelta.  This
+        # ensures that we get OverflowError rather than ValueError when
+        # seconds is too large.
         if seconds >= 0:
-            return datetime.datetime.fromtimestamp(0, tz=None) + datetime.timedelta(seconds=seconds, microseconds=microseconds)
+            return datetime.datetime.fromtimestamp(0, tz=datetime.timezone.utc) + datetime.timedelta(seconds=seconds, microseconds=microseconds)
         else:
-            return datetime.datetime.fromtimestamp(0, tz=None) - datetime.timedelta(seconds=-seconds, microseconds=microseconds)
+            return datetime.datetime.fromtimestamp(0, tz=datetime.timezone.utc) - datetime.timedelta(seconds=-seconds, microseconds=microseconds)
 
     {% when Type::Duration -%}
     # The Duration type.
@@ -129,24 +132,12 @@ class RustBufferStream(object):
 
     {% when Type::Object with (object_name) -%}
     # The Object type {{ object_name }}.
-    # Objects cannot currently be serialized, but we can produce a helpful error.
 
     def read{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.read not implemented yet for {{ canonical_type_name }}")
-
-    {% when Type::CallbackInterface with (object_name) -%}
-    # The Callback Interface type {{ object_name }}.
-    # Objects cannot currently be serialized, but we can produce a helpful error.
-
-    def read{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.read not implemented yet for {{ canonical_type_name }}")
-
-    {% when Type::Error with (error_name) -%}
-    # The Error type {{ error_name }}.
-    # Errors cannot currently be serialized, but we can produce a helpful error.
-
-    def read{{ canonical_type_name }}(self):
-        raise InternalError("RustBufferStream.read not implemented yet for {{ canonical_type_name }}")
+        # The Rust code always expects pointers written as 8 bytes,
+        # and will fail to compile if they don't fit in that size.
+        pointer = self._unpack_from(8, ">Q")
+        return {{ object_name|class_name_py }}._make_instance_(pointer)
 
     {% when Type::Enum with (enum_name) -%}
     {%- let e = ci.get_enum_definition(enum_name).unwrap() -%}
@@ -172,6 +163,35 @@ class RustBufferStream(object):
         raise InternalError("Unexpected variant tag for {{ canonical_type_name }}")
         {%- endif %}
 
+    {% when Type::Error with (error_name) -%}
+    {%- let e = ci.get_error_definition(error_name).unwrap().wrapped_enum() %}
+
+    # The Error type {{ error_name }}
+
+    # Top-level read method
+    def read{{ canonical_type_name }}(self):
+        variant = self._unpack_from(4, ">i")
+        try:
+            read_variant_method = getattr(self, 'readVariant{}Of{{canonical_type_name}}'.format(variant))
+        except AttributeError:
+            raise InternalError("Unexpected variant value for error {{ canonical_type_name }} ({})".format(variant))
+        return read_variant_method()
+
+    # Read methods for each individual variants
+    {%- for variant in e.variants() %}
+
+    def readVariant{{ loop.index}}Of{{ canonical_type_name }}(self):
+        {%- if variant.has_fields() %}
+        return {{ error_name|class_name_py }}.{{ variant.name()|class_name_py }}(
+            {%- for field in variant.fields() %}
+            self.read{{ field.type_().canonical_name()|class_name_py }}(),
+            {%- endfor %}
+        )
+        {%- else %}
+        return {{ error_name|class_name_py }}.{{ variant.name()|class_name_py }}()
+        {%- endif %}
+    {%- endfor %}
+
     {% when Type::Record with (record_name) -%}
     {%- let rec = ci.get_record_definition(record_name).unwrap() -%}
     # The Record type {{ record_name }}.
@@ -222,5 +242,10 @@ class RustBufferStream(object):
             count -= 1
         return items
 
+    {%- else -%}
+    # This type cannot currently be serialized, but we can produce a helpful error.
+    def read{{ canonical_type_name }}(self):
+        raise InternalError("RustBufferStream.read not implemented yet for {{ canonical_type_name }}")
+
     {%- endmatch -%}
     {%- endfor %}
