@@ -18,7 +18,7 @@ use std::convert::{TryFrom, TryInto};
 
 use anyhow::{bail, Result};
 
-/// Represents an attribute parsed from UDL, like [ByRef] or [Throws].
+/// Represents an attribute parsed from UDL, like `[ByRef]` or `[Throws]`.
 ///
 /// This is a convenience enum for parsing UDL attributes and erroring out if we encounter
 /// any unsupported ones. These don't convert directly into parts of a `ComponentInterface`, but
@@ -107,7 +107,7 @@ where
         .collect::<Result<Vec<_>, _>>()?;
 
     for attr in attrs.iter() {
-        validator(&attr)?;
+        validator(attr)?;
     }
 
     Ok(attrs)
@@ -237,6 +237,10 @@ impl InterfaceAttributes {
         self.0.iter().any(|attr| attr.is_enum())
     }
 
+    pub fn contains_error_attr(&self) -> bool {
+        self.0.iter().any(|attr| attr.is_error())
+    }
+
     pub fn threadsafe(&self) -> bool {
         self.0
             .iter()
@@ -251,6 +255,7 @@ impl TryFrom<&weedle::attribute::ExtendedAttributeList<'_>> for InterfaceAttribu
     ) -> Result<Self, Self::Error> {
         let attrs = parse_attributes(weedle_attributes, |attr| match attr {
             Attribute::Enum => Ok(()),
+            Attribute::Error => Ok(()),
             Attribute::Threadsafe => Ok(()),
             _ => bail!(format!("{:?} not supported for interface definition", attr)),
         })?;
@@ -330,7 +335,7 @@ impl MethodAttributes {
         })
     }
 
-    pub(super) fn get_by_arc(&self) -> bool {
+    pub(super) fn get_self_by_arc(&self) -> bool {
         self.0
             .iter()
             .any(|attr| matches!(attr, Attribute::SelfType(SelfType::ByArc)))
@@ -525,23 +530,23 @@ mod test {
     fn test_method_attributes() {
         let (_, node) = weedle::attribute::ExtendedAttributeList::parse("[Throws=Error]").unwrap();
         let attrs = MethodAttributes::try_from(&node).unwrap();
-        assert!(!attrs.get_by_arc());
+        assert!(!attrs.get_self_by_arc());
         assert!(matches!(attrs.get_throws_err(), Some("Error")));
 
         let (_, node) = weedle::attribute::ExtendedAttributeList::parse("[]").unwrap();
         let attrs = MethodAttributes::try_from(&node).unwrap();
-        assert!(!attrs.get_by_arc());
+        assert!(!attrs.get_self_by_arc());
         assert!(attrs.get_throws_err().is_none());
 
         let (_, node) =
             weedle::attribute::ExtendedAttributeList::parse("[Self=ByArc, Throws=Error]").unwrap();
         let attrs = MethodAttributes::try_from(&node).unwrap();
-        assert!(attrs.get_by_arc());
+        assert!(attrs.get_self_by_arc());
         assert!(attrs.get_throws_err().is_some());
 
         let (_, node) = weedle::attribute::ExtendedAttributeList::parse("[Self=ByArc]").unwrap();
         let attrs = MethodAttributes::try_from(&node).unwrap();
-        assert!(attrs.get_by_arc());
+        assert!(attrs.get_self_by_arc());
         assert!(attrs.get_throws_err().is_none());
     }
 
@@ -641,11 +646,11 @@ mod test {
     #[test]
     fn test_other_attributes_not_supported_for_interfaces() {
         let (_, node) =
-            weedle::attribute::ExtendedAttributeList::parse("[Threadsafe, Error]").unwrap();
+            weedle::attribute::ExtendedAttributeList::parse("[Threadsafe, ByRef]").unwrap();
         let err = InterfaceAttributes::try_from(&node).unwrap_err();
         assert_eq!(
             err.to_string(),
-            "Error not supported for interface definition"
+            "ByRef not supported for interface definition"
         );
     }
 }
