@@ -39,7 +39,7 @@ use anyhow::{bail, Result};
 
 use super::ffi::{FFIArgument, FFIFunction, FFIType};
 use super::object::Method;
-use super::types::Type;
+use super::types::{IterTypes, Type, TypeIterator};
 use super::{APIConverter, ComponentInterface};
 
 #[derive(Debug, Clone)]
@@ -84,6 +84,12 @@ impl CallbackInterface {
     }
 }
 
+impl IterTypes for CallbackInterface {
+    fn iter_types(&self) -> TypeIterator<'_> {
+        Box::new(self.methods.iter().map(IterTypes::iter_types).flatten())
+    }
+}
+
 impl Hash for CallbackInterface {
     fn hash<H: Hasher>(&self, state: &mut H) {
         // We don't include the FFIFunc in the hash calculation, because:
