@@ -60,7 +60,7 @@
 //!
 //! First you will need to install `uniffi-bindgen` on your system using `cargo install uniffi_bindgen`.
 //! Then add to your crate `uniffi_build` under `[build-dependencies]`.
-//! Finally, add a `build.rs` script to your crate and have it call [uniffi_build::generate_scaffolding](uniffi_build::generate_scaffolding)
+//! Finally, add a `build.rs` script to your crate and have it call `uniffi_build::generate_scaffolding`
 //! to process your `.udl` file. This will generate some Rust code to be included in the top-level source
 //! code of your crate. If your UDL file is named `example.udl`, then your build script would call:
 //!
@@ -125,14 +125,14 @@ pub fn generate_component_scaffolding<P: AsRef<Path>>(
     let config_file_override = config_file_override.as_ref().map(|p| p.as_ref());
     let out_dir_override = out_dir_override.as_ref().map(|p| p.as_ref());
     let udl_file = udl_file.as_ref();
-    let component = parse_udl(&udl_file)?;
+    let component = parse_udl(udl_file)?;
     let _config = get_config(&component, udl_file, config_file_override);
     let mut filename = Path::new(&udl_file)
         .file_stem()
         .ok_or_else(|| anyhow!("not a file"))?
         .to_os_string();
     filename.push(".uniffi.rs");
-    let mut out_dir = get_out_dir(&udl_file, out_dir_override)?;
+    let mut out_dir = get_out_dir(udl_file, out_dir_override)?;
     out_dir.push(filename);
     let mut f =
         File::create(&out_dir).map_err(|e| anyhow!("Failed to create output file: {:?}", e))?;
@@ -157,9 +157,9 @@ pub fn generate_bindings<P: AsRef<Path>>(
     let config_file_override = config_file_override.as_ref().map(|p| p.as_ref());
     let udl_file = udl_file.as_ref();
 
-    let component = parse_udl(&udl_file)?;
+    let component = parse_udl(udl_file)?;
     let config = get_config(&component, udl_file, config_file_override)?;
-    let out_dir = get_out_dir(&udl_file, out_dir_override)?;
+    let out_dir = get_out_dir(udl_file, out_dir_override)?;
     for language in target_languages {
         bindings::write_bindings(
             &config.bindings,
@@ -167,7 +167,6 @@ pub fn generate_bindings<P: AsRef<Path>>(
             &out_dir,
             language.try_into()?,
             try_format_code,
-            false,
         )?;
     }
     Ok(())
@@ -185,7 +184,7 @@ pub fn run_tests<P: AsRef<Path>>(
     let udl_file = udl_file.as_ref();
     let config_file_override = config_file_override.as_ref().map(|p| p.as_ref());
 
-    let component = parse_udl(&udl_file)?;
+    let component = parse_udl(udl_file)?;
     let config = get_config(&component, udl_file, config_file_override)?;
 
     // Group the test scripts by language first.
@@ -202,7 +201,7 @@ pub fn run_tests<P: AsRef<Path>>(
     }
 
     for (lang, test_scripts) in language_tests {
-        bindings::write_bindings(&config.bindings, &component, &cdylib_dir, lang, true, true)?;
+        bindings::write_bindings(&config.bindings, &component, &cdylib_dir, lang, true)?;
         bindings::compile_bindings(&config.bindings, &component, &cdylib_dir, lang)?;
         for test_script in test_scripts {
             bindings::run_script(cdylib_dir, &test_script, lang)?;
@@ -324,7 +323,7 @@ impl<T: Clone> MergeWith for Option<T> {
 }
 
 pub fn run_main() -> Result<()> {
-    const POSSIBLE_LANGUAGES: &[&str] = &["kotlin", "python", "swift", "gecko_js", "ruby"];
+    const POSSIBLE_LANGUAGES: &[&str] = &["kotlin", "python", "swift", "ruby"];
     let matches = clap::App::new("uniffi-bindgen")
         .about("Scaffolding and bindings generator for Rust")
         .version(clap::crate_version!())
@@ -339,7 +338,7 @@ pub fn run_main() -> Result<()> {
                         .short("-l")
                         .multiple(true)
                         .number_of_values(1)
-                        .possible_values(&POSSIBLE_LANGUAGES)
+                        .possible_values(POSSIBLE_LANGUAGES)
                         .help("Foreign language(s) for which to build bindings"),
                 )
                 .arg(
